/*******************************************************************************************/
/**                                                                                       **/
/** COPYRIGHT (C) 2010, Systemyde International Corporation, ALL RIGHTS RESERVED          **/
/**                                                                                       **/
/** display driver                                                    Rev 0.0  08/01/2011 **/
/**                                                                                       **/
/*******************************************************************************************/
module disp_top (data_bus, dpwo, isa_bus, ph1, ph2, pwo, pwr_up, sync);

  inout         data_bus;      /* memory/peripheral data bus                          bus  */
  input         isa_bus;       /* instruction address/data bus                        bus  */
  input         ph1;           /* clock phase 1                                       bus  */
  input         ph2;           /* clock phase 2                                       bus  */
  input         pwo;           /* power off                                           bus  */
  input         pwr_up;        /* power on init                                       brd  */
  input         sync;          /* synchronization                                     bus  */
  output        dpwo;          /* display power off                                   bus  */

  /*****************************************************************************************/
  /*                                                                                       */
  /* signal declarations                                                                   */
  /*                                                                                       */
  /*****************************************************************************************/
  wire        bit_pls;                                     /* conditional ld_cycle         */
  wire        bits_4,    bits_8,    bits_12;               /* recirculate strobes          */
  wire        bits_48x4, bits_48x8, bits_48x12;            /* recirculate strobes          */
  wire        dispoff_pls;                                 /* disp off instruction pulse   */
  wire        dispran_inst;                                /* read ann instruction         */
  wire        disptog_pls;                                 /* disp tog instruction pulse   */
  wire        dispwan_inst;                                /* write ann instruction        */
  wire        dpwo;                                        /* display power off            */
  wire        dread_inst;                                  /* display read instruction     */
  wire        dwrite_inst;                                 /* display write instruction    */
  wire        ld_cycle;                                    /* next cycle                   */
  wire        ld_dout;                                     /* load dout register           */
  wire        ld_iftch;                                    /* load serial inst             */
  wire        ld_inst;                                     /* latch instruction            */
  wire        pfadc_inst;                                  /* pfad=c instruction           */
  wire        pfadc_pls;                                   /* pfad=c instruction pulse     */
  wire        state_e, state_c, state_8, state_0;          /* decoded state                */
  wire        state_1, state_2, state_5, state_a;          /* decoded state                */
  wire        state_4, state_9, state_3, state_6;          /* decoded state                */
  wire        state_d, state_b;                            /* decoded state                */

  reg         c11_reg, c10_reg, c9_reg, c8_reg;            /* "c" display register         */
  reg         c7_reg,  c6_reg,  c5_reg, c4_reg;
  reg         c3_reg,  c2_reg,  c1_reg, c0_reg;
  reg         den_reg;                                     /* display enable               */
  reg         disp_shft;                                   /* shift enable                 */
  reg         dstat_reg;                                   /* display status               */
  reg         left_shft;                                   /* shift left                   */
  reg         peri_fd;                                     /* fd peripheral decode         */
  reg   [3:0] a0_in,  b0_in,  c0_in;                       /* display lsb in               */
  reg   [3:0] a11_in, b11_in, c11_in;                      /* display msb in               */
  reg   [3:0] a11_reg, a10_reg, a9_reg, a8_reg;            /* "a" display register         */
  reg   [3:0] a7_reg,  a6_reg,  a5_reg, a4_reg;
  reg   [3:0] a3_reg,  a2_reg,  a1_reg, a0_reg;
  reg   [3:0] b11_reg, b10_reg, b9_reg, b8_reg;            /* "b" display register         */
  reg   [3:0] b7_reg,  b6_reg,  b5_reg, b4_reg;
  reg   [3:0] b3_reg,  b2_reg,  b1_reg, b0_reg;
  reg   [3:0] cycle_reg;                                   /* clock cycle                  */
  reg   [3:0] state_reg;                                   /* machine state                */
  reg   [8:0] dout_ini, dout_nxt, dout_reg;                /* data out assembly            */
  reg   [9:0] iftch_reg;                                   /* instruction assembly         */
  reg   [9:0] lftch_reg;                                   /* latched instruction          */
  reg  [11:0] ann_reg;                                     /* annunciators                 */
  reg  [11:0] din_reg;                                     /* data in assembly             */

  /*****************************************************************************************/
  /*                                                                                       */
  /* external signal interface                                                             */
  /*                                                                                       */
  /*****************************************************************************************/
  assign data_bus = (dread_inst && !ph2) ? dout_reg[0] : 1'bz;
  assign dpwo     = pwo || !den_reg || dstat_reg;

  /*****************************************************************************************/
  /*                                                                                       */
  /* instruction latch                                                                     */
  /*                                                                                       */
  /*****************************************************************************************/
  assign ld_iftch = sync || state_6 || state_d || (state_b && |cycle_reg[1:0]);

  always @ (posedge ph1 or negedge pwo) begin
    if          (!pwo) iftch_reg <= 10'h000;
    else if (ld_iftch) iftch_reg <= (sync) ? {isa_bus, iftch_reg[9:1]} : 10'h000;
    end

  always @ (posedge ph2 or negedge pwo) begin
    if         (!pwo) lftch_reg <= 10'h000;
    else if (ld_inst) lftch_reg <= iftch_reg;
    end

  /*****************************************************************************************/
  /*                                                                                       */
  /* data input latch                                                                      */
  /*                                                                                       */
  /*****************************************************************************************/
  always @ (posedge ph1 or negedge pwo) begin
    if (!pwo) din_reg <= 12'h000;
    else      din_reg <= {data_bus, din_reg[11:1]};
    end

  /*****************************************************************************************/
  /*                                                                                       */
  /* bus state machine                                                                     */
  /*                                                                                       */
  /*****************************************************************************************/
  /*                                                                                       */
  /*   state_reg     eeeecccc88880000111122225555aaaa4444999933336666ddddbbbb              */
  /*   cycle_reg     12481248124812481248124812481248124812481248124812481248              */
  /*   ld_cycle      ...1...1...1...1...1...1...1...1...1...1...1...1...1...1              */
  /*   ld_inst       .......................................................1              */
  /*   sync          ............................................1111111111..              */
  /*                                                                                       */
  /*****************************************************************************************/
  assign ld_cycle = cycle_reg[3];
  assign ld_inst  = state_b && cycle_reg[3];

  always @ (posedge ph2 or negedge pwo) begin
    if (!pwo) cycle_reg <= 4'b0100;
    else      cycle_reg <= {( !cycle_reg[3]   && cycle_reg[2]),
                            (~|cycle_reg[3:2] && cycle_reg[1]),
                             ~|cycle_reg[3:1], cycle_reg[3]};
    end

  always @ (posedge ph2 or negedge pwo) begin
    if          (!pwo) state_reg <= 4'b1011;
    else if (ld_cycle) state_reg <= {(state_reg[2] || state_b), state_reg[1:0],
                                     ((state_reg[0] ^ !state_reg[3]) && !state_b)};
    end

  assign state_e    =  state_reg[3] &&  state_reg[2] &&  state_reg[1] && !state_reg[0];
  assign state_c    =  state_reg[3] &&  state_reg[2] && !state_reg[1] && !state_reg[0];
  assign state_8    =  state_reg[3] && !state_reg[2] && !state_reg[1] && !state_reg[0];
  assign state_0    = !state_reg[3] && !state_reg[2] && !state_reg[1] && !state_reg[0];
  assign state_1    = !state_reg[3] && !state_reg[2] && !state_reg[1] &&  state_reg[0];
  assign state_2    = !state_reg[3] && !state_reg[2] &&  state_reg[1] && !state_reg[0];
  assign state_5    = !state_reg[3] &&  state_reg[2] && !state_reg[1] &&  state_reg[0];
  assign state_a    =  state_reg[3] && !state_reg[2] &&  state_reg[1] && !state_reg[0];
  assign state_4    = !state_reg[3] &&  state_reg[2] && !state_reg[1] && !state_reg[0];
  assign state_9    =  state_reg[3] && !state_reg[2] && !state_reg[1] &&  state_reg[0];
  assign state_3    = !state_reg[3] && !state_reg[2] &&  state_reg[1] &&  state_reg[0];
  assign state_6    = !state_reg[3] &&  state_reg[2] &&  state_reg[1] && !state_reg[0];
  assign state_d    =  state_reg[3] &&  state_reg[2] && !state_reg[1] &&  state_reg[0];
  assign state_b    =  state_reg[3] && !state_reg[2] &&  state_reg[1] &&  state_reg[0];

  assign bit_pls    = ld_cycle && (dread_inst || dwrite_inst);
  assign bits_4     = bit_pls && state_e;
  assign bits_8     = bit_pls && state_c;
  assign bits_12    = bit_pls && state_8;
  assign bits_48x4  = bit_pls &&
                      (state_e || state_c || state_8 || state_0 || state_1 || state_2 ||
                       state_5 || state_a || state_4 || state_9 || state_3 || state_6);
  assign bits_48x8  = bit_pls &&
                      (state_c || state_0 || state_2 || state_a || state_9 || state_6);
  assign bits_48x12 = bit_pls && (state_8 || state_2 || state_4 || state_6);

  /*****************************************************************************************/
  /*                                                                                       */
  /* data output                                                                           */
  /*                                                                                       */
  /*****************************************************************************************/
  assign ld_dout = ld_inst && ~|(iftch_reg[5:0] ^ 6'b111000);

  always @ (iftch_reg or a0_reg or a11_reg or b0_reg or b11_reg or c0_reg or c11_reg) begin
    case (iftch_reg[9:6]) //synopsys parallel_case
      4'b0111,                                             /*          frsda               */
      4'b1100,                                             /*          frsdab              */
      4'b1110: dout_ini = {c0_reg,  b0_reg,  a0_reg};      /*          frsdabc             */
      4'b1000: dout_ini = {1'b0,    4'b0000, b0_reg};      /*          frsdb               */
      4'b0001,                                             /*          flldb               */
      4'b1011: dout_ini = {1'b0,    4'b0000, b11_reg};     /*          flsdb               */
      4'b1001: dout_ini = {1'b0,    4'b0000, c0_reg};      /*          frsdc               */
      4'b0010,                                             /*          flldc               */
      4'b0110: dout_ini = {1'b0,    4'b0000, c11_reg};     /*          flsdc               */
      default: dout_ini = {c11_reg, b11_reg, a11_reg};
      endcase
    end

  always @ (lftch_reg or a1_reg or a10_reg or b1_reg or b10_reg or c1_reg or c10_reg) begin
    case (lftch_reg[9:6]) //synopsys parallel_case
      4'b0111,                                             /*          frsda               */
      4'b1100,                                             /*          frsdab              */
      4'b1110: dout_nxt = {c1_reg,  b1_reg,  a1_reg};      /*          frsdabc             */
      4'b1000: dout_nxt = {1'b0,    4'b0000, b1_reg};      /*          frsdb               */
      4'b0001,                                             /*          flldb               */
      4'b1011: dout_nxt = {1'b0,    4'b0000, b10_reg};     /*          flsdb               */
      4'b1001: dout_nxt = {1'b0,    4'b0000, c1_reg};      /*          frsdc               */
      4'b0010,                                             /*          flldc               */
      4'b0110: dout_nxt = {1'b0,    4'b0000, c10_reg};     /*          flsdc               */
      default: dout_nxt = {c10_reg, b10_reg, a10_reg};
      endcase
    end

  always @ (posedge ph2 or negedge pwo) begin
    if (!pwo) dout_reg <= 9'h000;
    else      dout_reg <= (ld_dout)   ? dout_ini :
                          (disp_shft) ? dout_nxt : {1'b0, dout_reg[8:1]};
    end

  /*****************************************************************************************/
  /*                                                                                       */
  /* instruction decodes                                                                   */
  /*                                                                                       */
  /*****************************************************************************************/
  assign dispoff_pls  = ~|(lftch_reg ^ 10'b1011100000) && ld_inst;
  assign disptog_pls  = ~|(lftch_reg ^ 10'b1100100000) && ld_inst;
  assign pfadc_inst   = ~|(lftch_reg ^ 10'b1111110000);
  assign pfadc_pls    = ~|(lftch_reg ^ 10'b1111110000) && ld_inst;
  assign dispwan_inst = ~|(lftch_reg ^ 10'b1011110000) && den_reg;
  assign dispran_inst = ~|(lftch_reg ^ 10'b0101111000) && den_reg;
  assign dread_inst   = ~|(lftch_reg[5:0] ^ 6'b111000) && den_reg;
  assign dwrite_inst  = ~|(lftch_reg[5:0] ^ 6'b101000) && den_reg;

  /*****************************************************************************************/
  /*                                                                                       */
  /* display status                                                                        */
  /*                                                                                       */
  /*****************************************************************************************/
  always @ (posedge ph2 or negedge pwr_up) begin
    if                         (!pwr_up) dstat_reg <= 1'b0;
    else if (dispoff_pls || disptog_pls) dstat_reg <= (disptog_pls) ? !dstat_reg : 1'b0;
    end

  /*****************************************************************************************/
  /*                                                                                       */
  /* display enable                                                                        */
  /*                                                                                       */
  /*****************************************************************************************/
  always @ (posedge ph2 or negedge pwo) begin
    if                                   (!pwo) peri_fd <= 1'b0;
    else if (state_c && ld_cycle && pfadc_inst) peri_fd <= ~|(din_reg[11:4] ^ 8'hfd);
    end

  always @ (posedge ph2 or negedge pwo) begin
    if           (!pwo) den_reg <= 1'b1;
    else if (pfadc_pls) den_reg <= peri_fd;
    end

  /*****************************************************************************************/
  /*                                                                                       */
  /* display annunciators                                                                  */
  /*                                                                                       */
  /*****************************************************************************************/
  always @ (posedge ph2 or negedge pwo) begin
    if                                     (!pwo) ann_reg <= 12'h000;
    else if (state_8 && ld_cycle && dispwan_inst) ann_reg <= din_reg;
    end

  /*****************************************************************************************/
  /*                                                                                       */
  /* display operations                                                                    */
  /*                                                                                       */
  /*****************************************************************************************/
  /*                                                                                       */
  /*   code  inst    len  | shift on            |   c11  b11  a11  c0   b0   a0            */
  /*   0000  srlda    48  e c 8 0 1 2 5 a 4 9 3 6   c0   b0   d2   c1   b1   a1            */
  /*   0001  srldb    48  e c 8 0 1 2 5 a 4 9 3 6   c0   d2   a0   c1   b1   a1            */
  /*   0010  srldc    48  e c 8 0 1 2 5 a 4 9 3 6   d2   b0   a0   c1   b1   a1            */
  /*   0011  srldab   48    c   0   2   a   9   6   c0   d2   d1   c1   b1   a1            */
  /*   0100  srldabc  48      8     2     4     6   d2   d1   d0   c1   b1   a1            */
  /*   0101  slldab   48    c   0   2   a   9   6   c10  b10  a10  c11  d2   d1            */
  /*   0110  slldabc  48      8     2     4     6   c10  b10  a10  d2   d1   d0            */
  /*   0111  srsda    4   e                         c0   b0   d2   c1   b1   a1            */
  /*   1000  srsdb    4   e                         c0   d2   a0   c1   b1   a1            */
  /*   1001  srsdc    4   e                         d2   b0   a0   c1   b1   a1            */
  /*   1010  slsda    4   e                         c10  b10  a10  c11  b11  d2            */
  /*   1011  slsdb    4   e                         c10  b10  a10  c11  d2   a11           */
  /*   1100  srsdab   8     c                       c0   d2   d1   c1   b1   a1            */
  /*   1101  slsdab   8     c                       c10  b10  a10  c11  d2   d1            */
  /*   1110  srsdabc  12      8                     d2   d1   d0   c1   b1   a1            */
  /*   1111  slsdabc  12      8                     c10  b10  a10  d2   d1   d0            */
  /*                                                                                       */
  /*   0000  fllda    48  e c 8 0 1 2 5 a 4 9 3 6   c10  b10  a10  c11  b11  a11  a11      */
  /*   0001  flldb    48  e c 8 0 1 2 5 a 4 9 3 6   c10  b10  a10  c11  b11  a11  b11      */
  /*   0010  flldc    48  e c 8 0 1 2 5 a 4 9 3 6   c10  b10  a10  c11  b11  a11  c11      */
  /*   0011  flldab   48    c   0   2   a   9   6   c10  b10  a10  c11  b11  a11  ab11     */
  /*   0100  flldabc  48      8     2     4     6   c10  b10  a10  c11  b11  a11  abc11    */
  /*   0101                                                                                */
  /*   0110  flsdc    4   e                         c10  b10  a10  c11  b11  a11  c11      */
  /*   0111  frsda    4   e                         c0   b0   a0   c1   b1   a1   a0       */
  /*   1000  frsdb    4   e                         c0   b0   a0   c1   b1   a1   b0       */
  /*   1001  frsdc    4   e                         c0   b0   a0   c1   b1   a1   c0       */
  /*   1010  flsda    4   e                         c10  b10  a10  c11  b11  a11  a11      */
  /*   1011  flsdb    4   e                         c10  b10  a10  c11  b11  a11  b11      */
  /*   1100  frsdab   8     c                       c0   b0   a0   c1   b1   a1   ab0      */
  /*   1101  flsdab   8     c                       c10  b10  a10  c11  b11  a11  ab11     */
  /*   1110  frsdabc  12      8                     c0   b0   a0   c1   b1   a1   abc0     */
  /*   1111  flsdabc  12      8                     c10  b10  a10  c11  b11  a11  abc11    */
  /*                                                                                       */
  /*****************************************************************************************/
  always @ (lftch_reg or a1_reg or a11_reg or din_reg) begin
    casex ({lftch_reg[9:6], lftch_reg[4]}) //synopsys parallel_case
      5'b00000,                                            /* srlda                        */
      5'b00010,                                            /* srldb                        */
      5'b00100,                                            /* srldc                        */
      5'b00110,                                            /* srldab                       */
      5'b01000,                                            /* srldabc                      */
      5'b0111x,                                            /* srsda,   frsda               */
      5'b1000x,                                            /* srsdb,   frsdb               */
      5'b1001x,                                            /* srsdc,   frsdc               */
      5'b1100x,                                            /* srsdab,  frsdab              */
      5'b1110x: a0_in = a1_reg;                            /* srsdabc, frsdabc             */
      5'b10100: a0_in = din_reg[11:8];                     /* srsdc                        */
      5'b01010,                                            /* slldab                       */
      5'b11010: a0_in = din_reg[7:4];                      /* slsdab                       */
      5'b01100,                                            /* slldabc                      */
      5'b11110: a0_in = din_reg[3:0];                      /* slsdabc                      */
      default:  a0_in = a11_reg;
      endcase
    end

  always @ (lftch_reg or b1_reg or b11_reg or din_reg) begin
    casex ({lftch_reg[9:6], lftch_reg[4]}) //synopsys parallel_case
      5'b00000,                                            /* srlda                        */
      5'b00010,                                            /* srldb                        */
      5'b00100,                                            /* srldc                        */
      5'b00110,                                            /* srldab                       */
      5'b01000,                                            /* srldabc                      */
      5'b0111x,                                            /* srsda,   frsda               */
      5'b1000x,                                            /* srsdb,   frsdb               */
      5'b1001x,                                            /* srsdc,   frsdc               */
      5'b1100x,                                            /* srsdab,  frsdab              */
      5'b1110x: b0_in = b1_reg;                            /* srsdabc, frsdabc             */
      5'b01010,                                            /* slldab                       */
      5'b10110,                                            /* slsdb                        */
      5'b11010: b0_in = din_reg[11:8];                     /* slsdab                       */
      5'b01100,                                            /* slldabc                      */
      5'b11110: b0_in = din_reg[7:4];                      /* slsdabc                      */
      default:  b0_in = b11_reg;
      endcase
    end

  always @ (lftch_reg or c1_reg or c11_reg or din_reg) begin
    casex ({lftch_reg[9:6], lftch_reg[4]}) //synopsys parallel_case
      5'b00000,                                            /* srlda                        */
      5'b00010,                                            /* srldb                        */
      5'b00100,                                            /* srldc                        */
      5'b00110,                                            /* srldab                       */
      5'b01000,                                            /* srldabc                      */
      5'b0111x,                                            /* srsda,   frsda               */
      5'b1000x,                                            /* srsdb,   frsdb               */
      5'b1001x,                                            /* srsdc,   frsdc               */
      5'b1100x,                                            /* srsdab,  frsdab              */
      5'b1110x: c0_in = c1_reg;                            /* srsdabc, frsdabc             */
      5'b01100,                                            /* slldabc                      */
      5'b11110: c0_in = din_reg[11:8];                     /* slsdabc                      */
      default:  c0_in = c11_reg;
      endcase
    end

  always @ (lftch_reg or a0_reg or a10_reg or din_reg) begin
    casex ({lftch_reg[9:6], lftch_reg[4]}) //synopsys parallel_case
      5'b00010,                                            /* srldb                        */
      5'b00100,                                            /* srldc                        */
      5'b01111,                                            /*          frsda               */
      5'b1000x,                                            /* srsdb,   frsdb               */
      5'b1001x,                                            /* srsdc,   frsdc               */
      5'b11001,                                            /*          frsdab              */
      5'b11101: a11_in = a0_reg;                           /*          frsdabc             */
      5'b00000,                                            /* srlda                        */
      5'b01110: a11_in = din_reg[11:8];                    /* srsda                        */
      5'b00110,                                            /* srldab                       */
      5'b11000: a11_in = din_reg[7:4];                     /* srsdab                       */
      5'b01000,                                            /* srldabc                      */
      5'b11100: a11_in = din_reg[3:0];                     /* srsdabc                      */
      default:  a11_in = a10_reg;
      endcase
    end

  always @ (lftch_reg or b0_reg or b10_reg or din_reg) begin
    casex ({lftch_reg[9:6], lftch_reg[4]}) //synopsys parallel_case
      5'b00000,                                            /* srlda                        */
      5'b00100,                                            /* srldc                        */
      5'b0111x,                                            /* srsda,   frsda               */
      5'b10001,                                            /*          frsdb               */
      5'b1001x,                                            /* srsdc,   frsdc               */
      5'b11001,                                            /*          frsdab              */
      5'b11101: b11_in = b0_reg;                           /*          frsdabc             */
      5'b00010,                                            /* srldb                        */
      5'b00110,                                            /* srldab                       */
      5'b10000,                                            /* srsdb                        */
      5'b11000: b11_in = din_reg[11:8];                    /* srsdab                       */
      5'b01000,                                            /* srldabc                      */
      5'b11100: b11_in = din_reg[7:4];                     /* srsdabc                      */
      default:  b11_in = b10_reg;
      endcase
    end

  always @ (lftch_reg or c0_reg or c10_reg or din_reg) begin
    casex ({lftch_reg[9:6], lftch_reg[4]}) //synopsys parallel_case
      5'b00000,                                            /* srlda                        */
      5'b00010,                                            /* srldb                        */
      5'b00110,                                            /* srldab                       */
      5'b0111x,                                            /* srsda,   frsda               */
      5'b1000x,                                            /* srsdb,   frsdb               */
      5'b10011,                                            /*          frsdc               */
      5'b1100x,                                            /* srsdab,  frsdab              */
      5'b11101: c11_in = c0_reg;                           /*          frsdabc             */
      5'b00100,                                            /* srldc                        */
      5'b01000,                                            /* srldabc                      */
      5'b10010: c11_in = din_reg[11:8];                    /* srsdc                        */
      default:  c11_in = c10_reg;
      endcase
    end

  always @ (lftch_reg or bits_4 or bits_8 or bits_12 or bits_48x4 or bits_48x8 or
            bits_48x12) begin
    casex ({lftch_reg[9:6], lftch_reg[4]}) //synopsys parallel_case
      5'b0000x,                                            /* srlda,   fllda               */
      5'b0001x,                                            /* srldb,   flldb               */
      5'b0010x: disp_shft = bits_48x4;                     /* srldc,   flldc               */
      5'b0011x,                                            /* srldab,  flldab              */
      5'b01010: disp_shft = bits_48x8;                     /* slldab                       */
      5'b0100x,                                            /* srldabc, flldabc             */
      5'b01100: disp_shft = bits_48x12;                    /* slldabc                      */
      5'b01101,                                            /*          flsdc               */
      5'b0111x,                                            /* srsda,   frsda               */
      5'b1000x,                                            /* srsdb,   frsdb               */
      5'b1001x,                                            /* srsdc,   frsdc               */
      5'b1010x,                                            /* slsda,   flsda               */
      5'b1011x: disp_shft = bits_4;                        /* slsdb,   flsdb               */
      5'b1100x,                                            /* srsdab,  frsdab              */
      5'b1101x: disp_shft = bits_8;                        /* slsdab,  flsdab              */
      5'b1110x,                                            /* srsdabc, frsdabc             */
      5'b1111x: disp_shft = bits_12;                       /* slsdabc, flsdabc             */
      default:  disp_shft = 1'b0;
      endcase
    end

  always @ (lftch_reg) begin
    casex ({lftch_reg[9:6], lftch_reg[4]}) //synopsys parallel_case
      5'b00001,                                            /*          fllda               */
      5'b00011,                                            /*          flldb               */
      5'b00101,                                            /*          flldc               */   
      5'b00111,                                            /*          flldab              */
      5'b01001,                                            /*          flldabc             */
      5'b01010,                                            /* slldab                       */
      5'b0110x,                                            /* slldabc, flsdc               */
      5'b1010x,                                            /* slsda,   flsda               */
      5'b1011x,                                            /* slsdb,   flsdb               */
      5'b1101x,                                            /* slsdab,  flsdab              */
      5'b1111x: left_shft = 1'b1;                          /* slsdabc, flsdabc             */
      default:  left_shft = 1'b0;
      endcase
    end

  /*****************************************************************************************/
  /*                                                                                       */
  /* display data                                                                          */
  /*                                                                                       */
  /*****************************************************************************************/
  always @ (posedge ph2) begin
    if (disp_shft) begin
      c11_reg <= c11_in;
      b11_reg <= b11_in;
      a11_reg <= a11_in;
      c10_reg <= (left_shft) ? c9_reg : c11_reg;
      b10_reg <= (left_shft) ? b9_reg : b11_reg;
      a10_reg <= (left_shft) ? a9_reg : a11_reg;
      c9_reg  <= (left_shft) ? c8_reg : c10_reg;
      b9_reg  <= (left_shft) ? b8_reg : b10_reg;
      a9_reg  <= (left_shft) ? a8_reg : a10_reg;
      c8_reg  <= (left_shft) ? c7_reg : c9_reg;
      b8_reg  <= (left_shft) ? b7_reg : b9_reg;
      a8_reg  <= (left_shft) ? a7_reg : a9_reg;
      c7_reg  <= (left_shft) ? c6_reg : c8_reg;
      b7_reg  <= (left_shft) ? b6_reg : b8_reg;
      a7_reg  <= (left_shft) ? a6_reg : a8_reg;
      c6_reg  <= (left_shft) ? c5_reg : c7_reg;
      b6_reg  <= (left_shft) ? b5_reg : b7_reg;
      a6_reg  <= (left_shft) ? a5_reg : a7_reg;
      c5_reg  <= (left_shft) ? c4_reg : c6_reg;
      b5_reg  <= (left_shft) ? b4_reg : b6_reg;
      a5_reg  <= (left_shft) ? a4_reg : a6_reg;
      c4_reg  <= (left_shft) ? c3_reg : c5_reg;
      b4_reg  <= (left_shft) ? b3_reg : b5_reg;
      a4_reg  <= (left_shft) ? a3_reg : a5_reg;
      c3_reg  <= (left_shft) ? c2_reg : c4_reg;
      b3_reg  <= (left_shft) ? b2_reg : b4_reg;
      a3_reg  <= (left_shft) ? a2_reg : a4_reg;
      c2_reg  <= (left_shft) ? c1_reg : c3_reg;
      b2_reg  <= (left_shft) ? b1_reg : b3_reg;
      a2_reg  <= (left_shft) ? a1_reg : a3_reg;
      c1_reg  <= (left_shft) ? c0_reg : c2_reg;
      b1_reg  <= (left_shft) ? b0_reg : b2_reg;
      a1_reg  <= (left_shft) ? a0_reg : a2_reg;
      c0_reg  <= c0_in;
      b0_reg  <= b0_in;
      a0_reg  <= a0_in;
      end
    end

  endmodule
















